@@ -9,9 +9,11 @@ LENS_FILE_PATH = _BASEDIR / "dpl_xy_z1_elliptical.npz"
 
 
 class Camera:
-    def __init__(self, cam_id=0, output_shape=None):
+    def __init__(self, cam_id=0, output_shape=None, output_dir=None):
         self.cam_id = cam_id
         self.output_shape = output_shape or (1280, 800)
+        self.output_dir = output_dir or str(Path.cwd().resolve())
+        self.save_counter = 0
         self.set_capture_device()
         self.read_image_properties()
 
@@ -21,25 +23,41 @@ class Camera:
     def read_capture_device(self):
         ack, img = self.camera.read()
         if not ack:
-            raise ValueError('Image could not be read from device')        
+            raise ValueError("Image could not be read from device")
         return img
 
     def read_image_properties(self):
         img = self.read_capture_device()
         self.shape = img.shape[:2]
-    
+
     def show(self, image=None):
         image = image if image is not None else self.read_capture_device()
         image = cv2.resize(image, self.output_shape)
-        cv2.imshow('Face Lensing', image)
+        cv2.imshow("Face Lensing", image)
 
     def switch_capture_device(self):
-        self.cam_id = 1 - self.cam_id   
+        self.cam_id = 1 - self.cam_id
         self.release()
-        print(f'Switching to camera {self.cam_id}')
+        print(f"Switching to camera {self.cam_id}")
         self.set_capture_device()
         self.read_image_properties()
-    
+
+    def take_screenshot(self, img):
+        img_name = f"face_lensing_screenshot_{self.save_counter}.jpg"
+        img_path = str(Path(self.output_dir) / img_name)
+        cv2.putText(
+            img=img,
+            text="Made with Face Lensing - CNRS",
+            org=(30, self.shape[0] - 30),
+            fontFace=cv2.FONT_HERSHEY_SIMPLEX,
+            fontScale=0.8,
+            color=(240, 240, 240),
+            thickness=2,
+        )
+        cv2.imwrite(img_path, img)
+        self.save_counter += 1
+        print(f"Image written as {img_path}")
+
     def release(self):
         self.camera.release()
 
@@ -70,12 +88,13 @@ class Morphing:
 
         # Create coordinates mapping between the image and the lens
         centred_coords = (
-            np.indices(self.target_shape).reshape(2, -1) 
-            - img_center[:, None] 
-            + lens_center[:, None]).astype(int)
+            np.indices(self.target_shape).reshape(2, -1)
+            - img_center[:, None]
+            + lens_center[:, None]
+        ).astype(int)
 
-        centred_coords[0] = centred_coords[0].clip(0, self.shape[0]-1)
-        centred_coords[1] = centred_coords[1].clip(0, self.shape[1]-1)
+        centred_coords[0] = centred_coords[0].clip(0, self.shape[0] - 1)
+        centred_coords[1] = centred_coords[1].clip(0, self.shape[1] - 1)
 
         lens_coords = np.reshape(centred_coords, (2, *self.target_shape))
 
@@ -89,7 +108,21 @@ class Morphing:
 
     def apply(self, image):
         return np.asarray(
-            list(map(lambda p, q: image[self.dply[p, q], self.dplx[p, q]], self.Y, self.X)))
+            list(
+                map(
+                    lambda p, q: image[self.dply[p, q], self.dplx[p, q]], self.Y, self.X
+                )
+            )
+        )
+
+    def increase_effect(self):
+        if self.zoom > 0.005:
+            self.zoom -= 0.005
+        self.init_morphing()
+
+    def decrease_effect(self):
+        self.zoom += 0.005
+        self.init_morphing()
 
 
 def main(lens_file=LENS_FILE_PATH, cam_id=0, zoom=0.07):
@@ -98,34 +131,34 @@ def main(lens_file=LENS_FILE_PATH, cam_id=0, zoom=0.07):
     img_display = cam.read_capture_device()
 
     while True:
-        img_display[...] = morph.apply(
-            cam.read_capture_device()
-        )
+        img_display[...] = morph.apply(cam.read_capture_device())
         cam.show(img_display)
 
-        if cv2.waitKey(1) & 0xFF == ord('c'):
-            cam.switch_capture_device()
-            morph.set_target_shape(cam.shape)
-            img_display = cam.read_capture_device()
+        if keypress := cv2.waitKey(25):
+            if keypress == ord("c"):
+                cam.switch_capture_device()
+                morph.set_target_shape(cam.shape)
+                img_display = cam.read_capture_device()
 
-        if cv2.waitKey(1) & 0xFF == ord('j'):
-            print("augmente effet")
-            morph.zoom -= 0.005
-            morph.init_morphing()
+            if keypress == ord("+"):
+                print("Increasing lens effect")
+                morph.increase_effect()
 
-        if cv2.waitKey(1) & 0xFF == ord('k'):
-            print("diminue effet")
-            morph.zoom += 0.005
-            morph.init_morphing()
+            if keypress == ord("-"):
+                print("Decreasing lens effect")
+                morph.decrease_effect()
 
-        if cv2.waitKey(1) & 0xFF == ord('q'):
-            break
+            if keypress == ord(" "):
+                print("Taking a screenshot")
+                cam.take_screenshot(img_display)
+
+            if keypress == ord("q"):
+                print("Quitting")
+                break
 
-    # When everything done, release the capture
     cam.release()
     cv2.destroyAllWindows()
 
 
 if __name__ == "__main__":
     main()
-    
\ No newline at end of file
